/*
 * show-own-posts.js
 * https://github.com/savetheinternet/Tinyboard/blob/master/js/show-op.js
 *
 * Adds "(You)" to a name field when the post is yours. Update references as well.
 *
 * Released under the MIT license
 * Copyright (c) 2014 Marcin Łabanowski <marcin@6irc.net>
 *
 * Usage:
 *   $config['additional_javascript'][] = 'js/jquery.min.js';
 *   $config['additional_javascript'][] = 'js/ajax.js';
 *   $config['additional_javascript'][] = 'js/show-own-posts.js';
 *
 */


+function(){


  var update_own = function() {
    if ($(this).is('.you')) return;
  
    var thread = $(this).parents('[id^="thread_"]').first();
    if (!thread.length) {
      thread = $(this);
    }
  
    var board = thread.attr('data-board');
    var posts = JSON.parse(localStorage.own_posts || '{}');
  
    var id = $(this).attr('id').split('_')[1];
  
    if (posts[board] && posts[board].indexOf(id) !== -1) 
    {
        $(this).addClass('post-you');
        return true;
    }
  
    // Update references
    var bpost = this;
    var plinks = $(this).find('section.post-body:first a:not([rel="nofollow"])');
  
    plinks.each(function() {
      var postID;
  
      if(postID = $(this).text().match(/^>>(\d+)$/))
        postID = postID[1];
      else
        return;
  
      if (posts[board] && posts[board].indexOf(postID) !== -1) 
      {
        $(bpost).addClass('post-marker');
  
        if(plinks.length > 1)
          $(this).html($(this).html()+' (Вы)');
      }
    });
  };
  
  var update_all = function() {
    $('div[id^="thread_"], article.post').each(update_own);
  };
  
  var board = null;
  
  $(function() {
    board = $('input[name="board"]').first().val();
  
    update_all();
  });
  
  $(document).on('ajax_after_post', function(e, r) {
    var posts = JSON.parse(localStorage.own_posts || '{}');
    posts[board] = posts[board] || [];
    posts[board].push(r.id);
    localStorage.own_posts = JSON.stringify(posts);
  });
  
  $(document).on('new_post', function(e,post) {
    var $post = $(post);
    if ($post.is('article.post')) { // it's a reply
      $post.each(update_own);
    }
    else {
      $post.each(update_own); // first OP
      $post.find('article.post').each(update_own); // then replies
    }
  });
  
  
  
  }();
  